/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include "util.h"
#include "getopt.h"
#include "globalregistry.h"
#include "configfile.h"
#include "kis_netclient.h"

// Some system defaults
#define SYSCONF_LOC "/usr/local/etc/"
const char *config_base = "kismet_alertlogger.conf";

// Global registry of stuff
GlobalRegistry *globalreg = NULL;

// Bleh, globals.
int page_generate = 2;

void WriteAlertHTML(GlobalRegistry *globalreg, int clean) {
	FILE *outf = NULL;
	char err[1024];
	char *fname;

	vector<KismetClient::alert_info *> avec =
		globalreg->kisclient->FetchAlerts();

	fname = strdup(globalreg->kismetui_config->ExpandLogPath(globalreg->kismetui_config->FetchOpt("logtemplate"), "ALERT", "", 0, 1).c_str());
	
	if ((outf = fopen(fname, "w")) == NULL) {
		snprintf(err, 1024, "Unable to open '%s' for writing", fname);
		globalreg->messagebus->InjectMessage(err, MSGFLAG_FATAL);
		globalreg->fatal_condition = 1;
		free(fname);
		return;
	}

	free(fname);

	// Clean and exit here
	if (clean) {
		fprintf(outf, "<html><head>"
				"<meta http-equiv=\"refresh\" content=\"0;../intro.html\">"
				"</head></html>");
		return;
	}

	fprintf(outf, "<html><head>"
			"<link rel=\"stylesheet\" href=\"../kismet.css\" type=\"text/css\">\n"
			"<meta http-equiv=\"refresh\" content=\"%d\">\n", page_generate);
	fprintf(outf, "</head><body>");


	// Include the dynamic sorting js
	fprintf(outf, "<script language=\"javascript\" src=\"../sorttable.js\">"
			"</script>\n");

	fprintf(outf, "<script language=\"javascript\">");
	fprintf(outf, "var alrtab = new SortTable(\"alrtab\");");

	fprintf(outf, "alrtab.AddColumn(\"Time\", \"class=kismet_alert\", "
			"\"left\", \"\");");
	fprintf(outf, "alrtab.AddColumn(\"Alert\", \"class=kismet_alert\", "
			"\"left\", \"\");");
	fprintf(outf, "alrtab.AddColumn(\"BSSID\", \"class=kismet_alert\", "
			"\"left\", \"\");");
	fprintf(outf, "alrtab.AddColumn(\"Source\", \"class=kismet_alert\", "
			"\"left\", \"\");");
	fprintf(outf, "alrtab.AddColumn(\"Dest\", \"class=kismet_alert\", "
			"\"left\", \"\");");
	fprintf(outf, "alrtab.AddColumn(\"Ch\", \"class=kismet_alert\", "
			"\"center\", \"integer\");");
	fprintf(outf, "alrtab.AddColumn(\"Info\", \"class=kismet_alert\", " "\"left\", \"\");");

	for (unsigned int x = 0; x < avec.size(); x++) {
		fprintf(outf, "alrtab.AddLine(\"%.24s\", \"%s\", \"%s\", \"%s\", "
				"\"%s\", \"%s\", \"%d\", \"%s\");",
				ctime((const time_t *) &(avec[x]->alert_ts.tv_sec)),
				avec[x]->header.c_str(),
				avec[x]->bssid.Mac2String().c_str(),
				avec[x]->source.Mac2String().c_str(),
				avec[x]->dest.Mac2String().c_str(),
				avec[x]->other.Mac2String().c_str(),
				avec[x]->channel,
				avec[x]->text.c_str());
	}

	fprintf(outf, "SortRows(alrtab, 0);");

	fprintf(outf, "</script>\n");

	fprintf(outf, "<div class=\"texthunk\"><b class=\"bheader\">Alerts</b>"
			"<blockquote>\n");
	fprintf(outf, "<table class=\"kismet_alert\" width=\"100%\">");

	fprintf(outf, "<tr class=\"kismet_alert\">");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 0);\">Time</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 1);\">Alert</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 2);\">BSSID</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 3);\">Source</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 4);\">Dest</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 5);\">Other</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 6);\">Ch</a></th>");
	fprintf(outf, "<th class=\"kismet_alert\"><a class=\"kismet_alr_header\" "
			"href=\"javascript:SortRows(alrtab, 7);\">Info</a></th>");
	fprintf(outf, "</tr>\n");

	fprintf(outf, "<script language=\"javascript\">alrtab.WriteRows()</script>\n");

	fprintf(outf, "</table>");
	fprintf(outf, "</blockquote></div></body></html>\n");

	if (outf)
		fclose(outf);
}

void WriteNetworkHTML(GlobalRegistry *globalreg, int clean) {
	FILE *outf = NULL;
	char err[1024];
	char *fname;

	fname = strdup(globalreg->kismetui_config->ExpandLogPath(globalreg->kismetui_config->FetchOpt("logtemplate"), "NETWORK", "", 0, 1).c_str());
	
	if ((outf = fopen(fname, "w")) == NULL) {
		snprintf(err, 1024, "Unable to open '%s' for writing", fname);
		globalreg->messagebus->InjectMessage(err, MSGFLAG_FATAL);
		globalreg->fatal_condition = 1;
		free(fname);
		return;
	}

	free(fname);

	// Clean and exit here
	if (clean) {
		fprintf(outf, "<html><head>"
				"<meta http-equiv=\"refresh\" content=\"0;../intro.html\">"
				"</head></html>");
		return;
	}

	map<mac_addr, KismetClient::network_info *> netmap = 
		globalreg->kisclient->FetchNetworks();

	fprintf(outf, "<html><head>"
			"<link rel=\"stylesheet\" href=\"../kismet.css\" type=\"text/css\">\n"
			"<meta http-equiv=\"refresh\" content=\"%d\">\n", page_generate);

	// Include the dynamic sorting js
	fprintf(outf, "<script language=\"javascript\" src=\"../sorttable.js\">"
			"</script>\n");
	fprintf(outf, "<script language=\"javascript\" src=\"../sortcookie.js\">"
			"</script>\n");

	fprintf(outf, "<script language=\"javascript\">");
	fprintf(outf, "var nettab = new SortTable(\"nettab\");");
	fprintf(outf, "nettab.AddColumn(\"Last Seen\", \"class=kismet_network\", "
			"\"left\", \"\");");
	fprintf(outf, "nettab.AddColumn(\"BSSID\", \"class=kismet_network\", "
			"\"left\", \"\");");
	fprintf(outf, "nettab.AddColumn(\"Type\", \"class=kismet_network\", "
			"\"center\", \"\");");
	fprintf(outf, "nettab.AddColumn(\"SSID\", \"class=kismet_network\", "
			"\"left\", \"\");");
	fprintf(outf, "nettab.AddColumn(\"WEP\", \"class=kismet_network\", "
			"\"center\", \"\");");
	fprintf(outf, "nettab.AddColumn(\"Channel\", \"class=kismet_network\", "
			"\"center\", \"numeric\");");
	fprintf(outf, "nettab.AddColumn(\"Packets\", \"class=kismet_network\", "
			"\"center\", \"numeric\");");
	fprintf(outf, "nettab.AddColumn(\"Details\", \"class=kismet_network\", "
			"\"center\", \"\");");

	for (map<mac_addr, KismetClient::network_info *>::const_iterator nitr = 
		 netmap.begin(); nitr != netmap.end(); ++nitr) {
		string typestr;
		switch (nitr->second->type) {
			case 0:
				typestr = "AP";
				break;
			case 1:
				typestr = "Ad-Hoc";
				break;
			case 2:
				typestr = "Probe";
				break;
			case 3:
				typestr = "Turbocell";
				break;
			case 4:
				typestr = "Data";
				break;
			default:
				typestr = "Unknown";
				break;
		}

		fprintf(outf, "nettab.AddLine(\"%.24s\", \"%s\", \"%s\", \"%s\", \"%s\", "
				"\"%d\", \"%d\", "
				"\"<a href=\\\"kismet-NETDET.html#%s\\\">link</a>\");",
				ctime((const time_t *) &(nitr->second->lasttime)),
				nitr->second->bssid.Mac2String().c_str(),
				typestr.c_str(),
				nitr->second->ssid.c_str(),
				nitr->second->wep ? "Yes" : "No",
				nitr->second->channel,
				(nitr->second->llcpackets + nitr->second->datapackets),
				nitr->second->bssid.Mac2String().c_str());
	}

	fprintf(outf, "SortRows(nettab, SortByCookie('nettab'));");
	fprintf(outf, "</script>\n");

	fprintf(outf, "</head><body>");

	fprintf(outf, "<div class=\"texthunk\"><b class=\"bheader\">Networks</b>"
			"<blockquote>\n");
	fprintf(outf, "<table class=\"kismet_network\" width=\"100%\">");
	fprintf(outf, "<tr class=\"kismet_network\">");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 0); SortCookie('nettab', 0);\">"
			"Last Seen</a></th>");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 1); SortCookie('nettab', 1);\">"
			"BSSID</a></th>");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 2); SortCookie('nettab', 2);\">"
			"Type</a></th>");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 3); SortCookie('nettab', 3);\">"
			"SSID</a></th>");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 4); SortCookie('nettab', 4);\">"
			"WEP</a></th>");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 5); SortCookie('nettab', 5);\">"
			"Channel</a></th>");
	fprintf(outf, "<th class=\"kismet_network\"><a class=\"kismet_net_header\" "
			"href=\"javascript:SortRows(nettab, 6); SortCookie('nettab', 6);\">"
			"Packets</a></th>");
	fprintf(outf, "<th class=\"kismet_network\">Details</th>");
	fprintf(outf, "</tr>\n");

	fprintf(outf, "<script language=\"javascript\">nettab.WriteRows()</script>\n");

	fprintf(outf, "</table>\n");
	fprintf(outf, "</blockquote></div></body></html>\n");

	if (outf)
		fclose(outf);
}

void WriteStatusHTML(GlobalRegistry *globalreg, int clean) {
	FILE *outf = NULL;
	char err[1024];
	char *fname;

	fname = strdup(globalreg->kismetui_config->ExpandLogPath(globalreg->kismetui_config->FetchOpt("logtemplate"), "STATUS", "", 0, 1).c_str());
	
	if ((outf = fopen(fname, "w")) == NULL) {
		snprintf(err, 1024, "Unable to open '%s' for writing", fname);
		globalreg->messagebus->InjectMessage(err, MSGFLAG_FATAL);
		globalreg->fatal_condition = 1;
		free(fname);
		return;
	}

	free(fname);

	// Clean and exit here
	if (clean) {
		fprintf(outf, "<html><head>"
				"<meta http-equiv=\"refresh\" content=\"0;../intro_status.html\">"
				"</head></html>");
		return;
	}

	fprintf(outf, "<html><head>"
			"<link rel=\"stylesheet\" href=\"../kismet.css\" type=\"text/css\">\n"
			"<meta http-equiv=\"refresh\" content=\"%d\">\n", page_generate);
	fprintf(outf, "</head><body>");

	fprintf(outf, "<div class=\"texthunk\"><b class=\"bheader\">Status</b>"
			"<blockquote>\n");

	vector<string> status_vec = globalreg->kisclient->FetchStatus();

	for (unsigned int x = 0; x < status_vec.size(); x++) {
		fprintf(outf, "%s<br>\n", status_vec[x].c_str());
	}

	fprintf(outf, "</blockquote></div></body></html>\n");

	if (outf)
		fclose(outf);
}

void WriteNetworkDetailHTML(GlobalRegistry *globalreg, int clean) {
	FILE *outf = NULL;
	char err[1024];
	char *fname;

	fname = strdup(globalreg->kismetui_config->ExpandLogPath(globalreg->kismetui_config->FetchOpt("logtemplate"), "NETDET", "", 0, 1).c_str());
	
	if ((outf = fopen(fname, "w")) == NULL) {
		snprintf(err, 1024, "Unable to open '%s' for writing", fname);
		globalreg->messagebus->InjectMessage(err, MSGFLAG_FATAL);
		globalreg->fatal_condition = 1;
		free(fname);
		return;
	}

	free(fname);

	// Clean and exit here
	if (clean) {
		fprintf(outf, "<html><head>"
				"<meta http-equiv=\"refresh\" content=\"0;../intro.html\">"
				"</head></html>");
		return;
	}

	fprintf(outf, "<html><head>"
			"<link rel=\"stylesheet\" href=\"../kismet.css\" type=\"text/css\">\n"
			"<meta http-equiv=\"refresh\" content=\"%d\">\n", page_generate);
	fprintf(outf, "</head><body>");

	map<mac_addr, KismetClient::network_info *> netmap = 
		globalreg->kisclient->FetchNetworks();

	for (map<mac_addr, KismetClient::network_info *>::const_iterator x =
		 netmap.begin(); x != netmap.end(); ++x) {
		KismetClient::network_info *ni = x->second;
		string typestr;

		switch (ni->type) {
			case 0:
				typestr = "AP";
				break;
			case 1:
				typestr = "Ad-Hoc";
				break;
			case 2:
				typestr = "Probe";
				break;
			case 3:
				typestr = "Turbocell";
				break;
			case 4:
				typestr = "Data";
				break;
			default:
				typestr = "Unknown";
				break;
		}

		fprintf(outf, "<a name=\"%s\"</a>", x->first.Mac2String().c_str());
		fprintf(outf, "<div class=\"texthunk\"><b class=\"bheader\">%s/'%s'</b>"
				"<blockquote><table border=\"0\">", x->first.Mac2String().c_str(),
				ni->ssid.c_str());

		fprintf(outf, "<tr><td width=\"30%\"><b>BSSID</b></td>"
				"<td>%s</td></tr>", ni->bssid.Mac2String().c_str());
		fprintf(outf, "<tr><td width=\"30%\"><b>SSID</b></td>"
				"<td>%s</td></tr>", ni->ssid == "" ? "<i>Cloaked</i>" : 
				ni->ssid.c_str());
		fprintf(outf, "<tr><td width=\"30%\"><b>Cloaked</b></td>"
				"<td>%s</td></tr>", ni->cloaked ? "No" : "Yes");
				fprintf(outf, "<tr><td width=\"30%\"><b>Type</b></td>"
				"<td>%s</td></tr>", typestr.c_str());
		fprintf(outf, "<tr><td width=\"30%\"><b>Beacon Info</b></td>"
				"<td>%s</td></tr>", ni->beaconinfo == " " ? "<i>None</i>" : 
				ni->beaconinfo.c_str());
		fprintf(outf, "<tr><td width=\"30%\"><b>Channel</b></td>"
				"<td>%d</td></tr>", ni->channel);
		fprintf(outf, "<tr><td width=\"30%\"><b>WEP</b></td>"
				"<td>%s</td></tr>", ni->wep ? "Yes" : "No");
		fprintf(outf, "<tr><td width=\"30%\"><b>Decrypted</b></td>"
				"<td>%s</td></tr>", ni->decrypted ? "Yes" : "No");

		fprintf(outf, "<tr><td width=\"30%\"><b>First Time</b></td>"
				"<td>%.24s</td></tr>", ctime((const time_t *) &(ni->firsttime)));
		fprintf(outf, "<tr><td width=\"30%\"><b>Last Time</b></td>"
				"<td>%.24s</td></tr>", ctime((const time_t *) &(ni->lasttime)));

		fprintf(outf, "<tr><td width=\"30%\"><b>LLC Packets</b></td>"
				"<td>%d</td></tr>", ni->llcpackets);
		fprintf(outf, "<tr><td width=\"30%\"><b>Data Packets</b></td>"
				"<td>%d</td></tr>", ni->datapackets);
		fprintf(outf, "<tr><td width=\"30%\"><b>Crypt Packets</b></td>"
				"<td>%d</td></tr>", ni->cryptpackets);
		fprintf(outf, "<tr><td width=\"30%\"><b>Weak Packets</b></td>"
				"<td>%d</td></tr>", ni->weakpackets);

		fprintf(outf, "<tr><td width=\"30%\"><b>Signal</b></td>"
				"<td>%d</td></tr>", ni->signal.c_str());
		fprintf(outf, "<tr><td width=\"30%\"><b>Noise</b></td>"
				"<td>%d</td></tr>", ni->noise.c_str());


		fprintf(outf, "</table></blockquote></div>"
				"<div class=\"topbar\"><a href=\"kismet-NETWORK.html\">Return</a>"
				"</div><br>\n");
	}

	fprintf(outf, "</body></html>\n");

	if (outf)
		fclose(outf);
}

void WriteInfoHTML(GlobalRegistry *globalreg, int clean) {
	FILE *outf = NULL;
	char err[1024];
	char *fname;

	fname = strdup(globalreg->kismetui_config->ExpandLogPath(globalreg->kismetui_config->FetchOpt("logtemplate"), "INFO", "", 0, 1).c_str());
	
	if ((outf = fopen(fname, "w")) == NULL) {
		snprintf(err, 1024, "Unable to open '%s' for writing", fname);
		globalreg->messagebus->InjectMessage(err, MSGFLAG_FATAL);
		globalreg->fatal_condition = 1;
		free(fname);
		return;
	}

	free(fname);

	// Clean and exit here
	if (clean) {
		fprintf(outf, "<html><head>"
				"<meta http-equiv=\"refresh\" content=\"0;../intro_info.html\">"
				"</head></html>");
		return;
	}

	fprintf(outf, "<html><head>"
			"<link rel=\"stylesheet\" href=\"../kismet.css\" type=\"text/css\">\n"
			"<meta http-equiv=\"refresh\" content=\"%d\">\n", page_generate);
	fprintf(outf, "</head><body>");

	if (globalreg->kisclient->Valid() == 0) {
		fprintf(outf, "<code><b>Not connected to Kismet server...</b></code>");
	} else {
		time_t rst = globalreg->kisclient->FetchRemoteStart();

		fprintf(outf, "<code><b>Kismet</b>: %s &quot;%s&quot; @ %0.24s | "
				"<b>Networks</b>: %d | <b>Packets</b>: %d | <b>Encrypted</b>: %d | "
				"<b>FMS Weak</b>: %d | <b>Packets/Sec</b>: %d</code>",
				globalreg->kisclient->FetchRemoteVersion().c_str(),
				globalreg->kisclient->FetchRemoteServername().c_str(),
				ctime((const time_t *) &rst),
				globalreg->kisclient->FetchNumNetworks(),
				globalreg->kisclient->FetchNumPackets(),
				globalreg->kisclient->FetchNumCryptPackets(),
				globalreg->kisclient->FetchNumWeakPackets(),
				globalreg->kisclient->FetchPacketrate());
	}

	fprintf(outf, "</body></html>\n");

	if (outf)
		fclose(outf);
}

// Core timer event that writes alerts out
int WriterEvent(Timetracker::timer_event *evt, void *parm,
					 GlobalRegistry *globalreg) {

	WriteAlertHTML(globalreg, 0);
	WriteNetworkHTML(globalreg, 0);
	WriteNetworkDetailHTML(globalreg, 0);
	WriteStatusHTML(globalreg, 0);
	WriteInfoHTML(globalreg, 0);

	sync();

	globalreg->messagebus->InjectMessage("Wrote network data...", MSGFLAG_INFO);
	
	return 1;
}

void CleanDynamic() {
	fprintf(stderr, "Cleaning dynamic HTML files...\n");

	WriteAlertHTML(globalreg, 1);
	WriteNetworkHTML(globalreg, 1);
	WriteNetworkDetailHTML(globalreg, 1);
	WriteStatusHTML(globalreg, 1);
	WriteInfoHTML(globalreg, 1);

	sync();
}

void Usage(char *argv) {
	fprintf(stderr, "%s [ -f configfile ]\n", argv);

	exit(1);
}

void CatchShutdown(int reason) {
	fprintf(stderr, "FATAL - Shutting down.\n");

	CleanDynamic();
	
	exit(1);
}

int main(int argc, char *argv[]) {
	char status[1024];

	static struct option long_options[] = {
		{ "config-file", required_argument, 0, 'f' },
	};

	char *configfile = NULL;

	int option_index;

	while (1) {
		int r = getopt_long(argc, argv, "f:o:",
							long_options, &option_index);

		if (r < 0)
			break;

		switch (r) {
			case 'f':
				configfile = optarg;
				break;

			default:
				Usage(argv[0]);
				break;
		}
	}
	
	// Build internal structures
	globalreg = new GlobalRegistry;
	
	// Create the messagebus and clients
	globalreg->messagebus = new MessageBus;
	StdoutMessageClient *stdmsgcli = new StdoutMessageClient(globalreg);
	globalreg->messagebus->RegisterClient(stdmsgcli, MSGFLAG_ALL);

	// Allocate a timetracker
	globalreg->timetracker = new Timetracker(globalreg);

	// Config file has to be opened before kisclient is created or it 
	// will be very sad.
	int freeconf = 0;
	if (configfile == NULL) {
		configfile = (char *) malloc(1024 * sizeof(char));
		snprintf(configfile, 1024, "%s/%s", SYSCONF_LOC, config_base);
		freeconf = 1;
	}

	ConfigFile *conf = new ConfigFile;

	// Parse the config file
	if (conf->ParseConfig(configfile) < 0) {
		exit(1);
	}

	// Grab the output file name
	if (conf->FetchOpt("logtemplate") == "") {
		globalreg->messagebus->InjectMessage("No log template in config file.", 
											 MSGFLAG_FATAL);
		globalreg->fatal_condition = 1;
		CatchShutdown(-1);
	}

	// Grab the page refresh rate
	if (conf->FetchOpt("pagegenerate") != "") {
		if (sscanf(conf->FetchOpt("pagegenerate").c_str(), "%d", 
				   &page_generate) != 1) {
			globalreg->messagebus->InjectMessage("Invalid pagegenerate in config "
												 "file.", MSGFLAG_FATAL);
			globalreg->fatal_condition = 1;
			CatchShutdown(-1);
		}
	}

	// Assign it to the global registry
	globalreg->kismetui_config = conf;

	// Create the network client
	KismetClient *kis_cli = new KismetClient(globalreg);
	globalreg->kisclient = kis_cli;


	// Register a timer event for our alert printer
	globalreg->timetracker->RegisterTimer(SERVER_TIMESLICES_SEC * page_generate,
										  NULL, 1, &WriterEvent, NULL);

	// Catch signals
	signal(SIGINT, CatchShutdown);
	signal(SIGTERM, CatchShutdown);
	signal(SIGHUP, CatchShutdown);

	while (1) {
		int max_fd = 0;
		fd_set rset, wset;
		struct timeval tm;

		FD_ZERO(&rset);
		FD_ZERO(&wset);

		max_fd = globalreg->kisclient->MergeSet(rset, wset, max_fd, &rset, &wset);

		tm.tv_sec = 0;
		tm.tv_usec = 100000;

		if (select(max_fd + 1, &rset, &wset, NULL, &tm) < 0) {
			if (errno != EINTR) {
				snprintf(status, STATUS_MAX, 
						 "select() error (%s)", strerror(errno));
				globalreg->messagebus->InjectMessage(status, MSGFLAG_FATAL);
				CatchShutdown(-1);
			}

		}

		// Only bail on fully fatal errors
		if (globalreg->kisclient->Poll(rset, wset) < 0 &&
			globalreg->fatal_condition)
			CatchShutdown(-1);

		globalreg->timetracker->Tick();

	}
	
}

