/*
    This file is part of Kismet

    Kismet is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Kismet is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kismet; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __KISNETCLIENT_H__
#define __KISNETCLIENT_H__

#include "config.h"

#include "clinetframework.h"
#include "tcpclient.h"
#include "macaddr.h"

#define CLIENT_NOTIFY  2
#define CLIENT_ALERT   4

class KismetClient : public ClientFramework {
public:
    KismetClient();
    KismetClient(GlobalRegistry *in_globalreg);
    virtual ~KismetClient();

    // Hooks so we can override straight to the TCP core
    virtual unsigned int MergeSet(fd_set in_rset, fd_set in_wset, 
                                  unsigned int in_max_fd,
                                  fd_set *out_rset, fd_set *out_wset) {
        return netclient->MergeSet(in_rset, in_wset, in_max_fd,
                                   out_rset, out_wset);
    }

    virtual int Poll(fd_set& in_rset, fd_set& in_wset) {
        return netclient->Poll(in_rset, in_wset);
    }
    
    virtual int ParseData();
    virtual int KillConnection();

    virtual int Reconnect();
    
    virtual int Shutdown();

	virtual int Valid();

    typedef struct alert_info {
        timeval alert_ts;
		mac_addr bssid;
		mac_addr source;
		mac_addr dest;
		mac_addr other;
		int channel;
		string header;
		string text;
    };

	// Network types
	enum wireless_network_type {
		network_ap,
		network_adhoc,
		network_probe,
		network_turbocell,
		network_data,
		network_remove
	};

	// Simplified network tracker
	typedef struct network_info {
		mac_addr bssid;
		int type;
		string ssid;
		string beaconinfo;
		int llcpackets, datapackets, cryptpackets, weakpackets;
		int channel;
		int wep;
		time_t firsttime, lasttime;
		int atype;
		string rangeip;
		string gpsfixed, minlat, minlon, minalt, minspd, 
			   maxlat, maxlon, maxalt, maxspd,
			   bestlat, bestlon, bestalt,
			   agglat, agglon, aggalt, aggpoints;
		string quality, signal, noise, bestquality, bestsignal, bestnoise;
		int cloaked, decrypted;
		int beaconrate;
		double maxrate; 
		int carrierset, maxseenrate, encodingset;
		unsigned long datasize;
	};

    // Enable a protocol - this lets clients based off our code pick what 
	// protocols they support.
    void EnableProtocol(string in_protocol);
    // Disable a protocol
    void RemoveProtocol(string in_protocol);

    // Fetch the time reported by the server
    time_t FetchServTime();

    vector <alert_info *> FetchAlerts() { return alerts; }
    void ClearAlerts() { alerts.clear(); }

	vector<string> FetchStatus() { return status_vec; }

	map<mac_addr, network_info*> FetchNetworks() { return network_map; }

	int FetchNumNetworks() { return n_networks; }
	int FetchNumPackets() { return n_packets; }
	int FetchNumCryptPackets() { return n_crypt; }
	int FetchNumWeakPackets() { return n_weak; }
	int FetchPacketrate() { return packetrate; }
	time_t FetchRemoteStart() { return remote_starttime; }
	string FetchRemoteVersion() { return remote_version; }
	string FetchRemoteServername() { return remote_servername; }

    void SendRaw(const char *in_cmd);
protected:
    TcpClient *tcpcli;

	int reconid;

    int cmdid;
    
    char host[MAXHOSTNAMELEN];
    int port;
    int reconnect_attempt;
    time_t last_disconnect;

    int InjectCommand(string in_cmd);

    // Has the server identified itself w/ a valid KISMET line?
    int server_identified;
    
    unsigned int maxalerts;

	// Alert vector
    vector<alert_info *> alerts;

	// Map of networks... No masked MACs so it can be a normal map
	map<mac_addr, network_info *> network_map;	

    // Protocols we have enabled so that we can replay them on a reconnect
    map<string, int> protocol_map;

    // Fields we enable (what we know how to parse)
    map<string, string> protocol_default_map;

	// Store text and error data
	int maxstatus;
	vector<string> status_vec;

	int n_networks, n_packets, n_crypt, n_weak, packetrate;

	string remote_version;
	time_t remote_starttime;
	string remote_servername;

};

#endif

